/* 
 * Generic IR module reader using timer interrupts
 * 
 * V1.1  
 * 
 * By: Lars Ivar Miljeteig, January 2013
 * Platform: AVR ATmega328p (avr-gcc)
 * Dependencies: 
 *
 */

#include "ir_generic.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdbool.h>

#ifndef PD3
#define PD3 3
#endif 

pulsestruct pulse;

struct ircontrol
{
  // Pulse time management
  volatile uint16_t overflow;
  volatile uint8_t compaVal;

  // Flag 
  volatile uint8_t done;
  
  volatile bool khzActive;
} 
ir;

void setKhzActive(bool active)
{
  if(active)
  {
    TIMSK1 = (1<<OCIE1A);
    ir.khzActive = true;
  }
  else
  {
    TIMSK1 = 0;
    ir.khzActive = false;
    PORTD |= (1<<PD3); // LED Off
  }
}

void toggleKhzActive()
{
  if(ir.khzActive == true)
  {
    setKhzActive(false);
  }
  else
  {
    setKhzActive(true);
  }
}

void ir_reset()
{
  ir.done = 1;
  TIMSK2 = 0;
  pulse.index = 0;   

  ir.overflow = 0;
  ir.compaVal = 0;
  
  setKhzActive(false);
  PORTD |= (1<<PD3);  // LED Off
}

void nextSample()
{
  if(++pulse.index >= pulse.length)
  {
    ir_reset();
  }

  ir.overflow = pulse.data[pulse.index] / 255;
  ir.compaVal = pulse.data[pulse.index] - ir.overflow * 255;
}

ISR(TIMER2_COMPA_vect)
{
  TCNT2 = 0;
  
  if (ir.compaVal == 0 && ir.overflow == 0)
  {
    // Toggle output pin
    toggleKhzActive();

    nextSample();
  }

  if (ir.overflow == 0)
  {
    OCR2A = ir.compaVal;
    ir.compaVal = 0;
  }
  else
  {
    --ir.overflow;
    OCR2A = 255;
  }    
}

ISR(TIMER1_COMPA_vect)
{
  PORTD ^= (1<<PD3);
  TCNT1H = 0;  
  TCNT1L = 0;  
}

void ir_init()
{
  // Use normal timer mode, no PWM or fancy stuff enabled
  TCCR2A = 0;

  // Timer2 prescaler, 16MHz / 128
  TCCR2B = (1<<CS22)|(1<<CS20);

  // Reset variables
  ir_reset();

  uint8_t i = 0;
  for(i=0; i<127; i++)
  {
    pulse.data[i] = 0;
  }
  
  TCCR1A = 0;
  TCCR1B = (1<<CS10);
  
  // 16000000 / 38000 = 421 tics/period
  // Since we have two pin changes per period, we get
  // 421 / 2 = 210 tics/pin change
  // Then measuring and tweaking gave 210 => 181
  // Probably caused by some delay in processing the code
  OCR1AL = 192;//181; // 187; // 37kHz 
  OCR1AH = 0;
  
  // Enable pin out
  DDRD |= (1<<PD3);
  
  setKhzActive(false);
  PORTD |= (1<<PD3); // LED Off
}

void ir_send()
{
  ir.done = 0;

  nextSample();
  
  // Enable timer compare interrupt A
  TCNT2 = 0;
  TIMSK2 |= (1<<OCIE2A);
  setKhzActive(true);
}









